# CalibrationManager
Base classes: DeviceClientBase, Device
## <a name='interfaces'></a>Device interfaces (`interfaces`)
Type
: VECTOR_STRING

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: Interfaces implemented by this device.

Default value
: ``['DeviceInstantiator']``

## <a name='detectorType'></a>Detector type (`detectorType`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Type of the detector to manage.

Options:
: ``AGIPD``, ``LPD``, ``DSSC``, ``Jungfrau``, ``ePix100``, ``pnCCD``, ``FastCCD``

## <a name='detectorIdentifier'></a>Detector identifier (`detectorIdentifier`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Name of this detector in CalCat and device ID domain for the corresponding data aggregators, if applicable. 

## <a name='daqDevicesPattern'></a>DAQ devices pattern (`daqDevicesPattern`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Regexp pattern for the DAQ data aggregators of this detector, the formatting placeholder 'detector_identifier' may be used. Leave empty to disable DAQ interaction.

Default value
: ``^{detector_identifier}/DET/\d*CH0$``

## Class IDs (`classIds`)
class IDs to instantiate for each role. A formatting placeholer may be used to indicate the capitalized detector type, e.g. Dssc or Jungfrau
### <a name='classIds.correctionClass'></a>Correction class (`classIds.correctionClass`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Device class to use for corrections.

Default value
: ``{}Correction``

### <a name='classIds.groupMatcherClass'></a>Group matcher class (`classIds.groupMatcherClass`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Device class to use for matching the stream output of a module group.

Default value
: ``ShmemTrainMatcher``

### <a name='classIds.assemblerClass'></a>Assembler class (`classIds.assemblerClass`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Device class to use for assembling the matched output of a preview layer.

Default value
: ``DetectorAssembler``

## <a name='classIds'></a>Class IDs (`classIds`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: class IDs to instantiate for each role. A formatting placeholer may be used to indicate the capitalized detector type, e.g. Dssc or Jungfrau

## Device IDs (`deviceIds`)
Templates for the member field of the device IDs in the pipeline, which is appended to the same device ID root (DOMAIN/TYPE) as the manager instance itself.
### <a name='deviceIds.correctionSuffix'></a>Correction suffix (`deviceIds.correctionSuffix`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Suffix for correction device IDs. The formatting placeholders 'virtualName', 'index' and 'group' may be used.

Default value
: ``CORRECT{index:02d}_{virtualName}``

### <a name='deviceIds.groupMatcherSuffix'></a>Group matcher suffix (`deviceIds.groupMatcherSuffix`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Suffix for group matching device IDs. The formatting placeholder 'group' may be used.

Default value
: ``MATCH_G{group}``

### <a name='deviceIds.assemblerSuffix'></a>Assembler suffix (`deviceIds.assemblerSuffix`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Suffix for assembler device IDs. The formatting placeholder 'layer' may be used.

Default value
: ``ASSEMBLE_{layer}``

## <a name='deviceIds'></a>Device IDs (`deviceIds`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Templates for the member field of the device IDs in the pipeline, which is appended to the same device ID root (DOMAIN/TYPE) as the manager instance itself.

## <a name='modules'></a>Modules (`modules`)
Type
: VECTOR_HASH

Access mode
: RECONFIGURABLE

Assignment
: MANDATORY

Description
: Individual modules constituting this detector. A module is the boundary on which correction constants are calculated, and in general equivalent to PDUs. Only the virtual name and group columns are obligatory, if left blank the aggregator name is set to f'{detector_type.upper()}{index:02d}', the input channel is set to f'{detector_identifier}/DET/{index}CH0:output' and the input source f'{detector_identifier}/DET/{index}CH0:xtdf'.

Table schema and default value
: 
| Virtual name | Group | Aggregator name | Input channel | Input source |
| - | - | - | - | - |


## <a name='moduleGroups'></a>Module groups (`moduleGroups`)
Type
: VECTOR_HASH

Access mode
: RECONFIGURABLE

Assignment
: MANDATORY

Table schema and default value
: 
| Group | Device server | Include matcher? | Start matcher? | Include bridge? | Bridge port | Bridge pattern |
| - | - | - | - | - | - | - |


## <a name='previewLayers'></a>Preview layers (`previewLayers`)
Type
: VECTOR_HASH

Access mode
: RECONFIGURABLE

Assignment
: MANDATORY

Table schema and default value
: 
| Preview layer | Output pipeline | Device server |
| - | - | - |


## <a name='deviceServers'></a>Device servers (`deviceServers`)
Type
: VECTOR_HASH

Access mode
: RECONFIGURABLE

Assignment
: MANDATORY

Table schema and default value
: 
| Device server | Webserver host |
| - | - |


## <a name='geometryDevice'></a>Geometry device (`geometryDevice`)
Type
: STRING

Access mode
: INITONLY

Assignment
: MANDATORY

Description
: Device ID for a geometry device defining the detector layout and module positions.

## <a name='maxIdle'></a>Preview source timeout (`maxIdle`)
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: TrainMatcher (used for preview assemblers) has an option (maxIdle) to automatically ignore sources which have not sent data in the last maxIdle seconds (if maxIdle > 0). Useful in case a module fails. If missing source comes back, it will be matched again.

Default value
: ``1.0``

## Webserver API (`webserverApi`)
Configurations for the webserver API to control device servers.
### <a name='webserverApi.statePollInterval'></a>Status poll interval (`webserverApi.statePollInterval`)
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Time between subsequent polls of the webserver API when a device server is expected to reach a particular state.

Default value
: ``0.5``

### <a name='webserverApi.downTimeout'></a>Down timeout (`webserverApi.downTimeout`)
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Time to wait for a device server to stop gracefully after a 'down' command and until the 'kill' command is sent.

Default value
: ``5.0``

### <a name='webserverApi.killTimeout'></a>Kill timeout (`webserverApi.killTimeout`)
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Time to wait for a device server to stop after a 'kill' command and until the server is deemed unrecoverable.

Default value
: ``5.0``

### <a name='webserverApi.upTimeout'></a>Up timeout (`webserverApi.upTimeout`)
Type
: DOUBLE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Time to wait for a device server to start successfully after being down and until the server i deemed broken.

Default value
: ``5.0``

## <a name='webserverApi'></a>Webserver API (`webserverApi`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Configurations for the webserver API to control device servers.

## <a name='managedKeyConfiguration'></a>Managed key configuration (`managedKeyConfiguration`)
Type
: STRING

Access mode
: INITONLY

Assignment
: OPTIONAL

Description
: Name of manager device configuration to apply to managed keys on startup, leave empty to disable.

Default value
: empty string

## <a name='restoredConfigurations'></a>Restored configurations (`restoredConfigurations`)
Type
: VECTOR_HASH

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Regexp patterns matched against instantiated devices to selectively restore the most previous configuration.

Table schema and default value
: 
| Enabled | Member pattern | Key pattern |
| - | - | - |
| ``False`` | ``MATCH_G\d`` | ``^(channels|fastSources|slowSources)`` |


## <a name='restartServers'></a>Restart servers (`restartServers`)
Type
: Slot

Allowed in states
: ACTIVE, ERROR

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## <a name='startInstantiate'></a>Instantiate pipeline (`startInstantiate`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

## <a name='managedDevices'></a>Managed devices (`managedDevices`)
Type
: VECTOR_STRING

Access mode
: READONLY

Assignment
: OPTIONAL

Description
: List of currently managed devices.

Default value
: ``[]``

## <a name='discoverManagedDevices'></a>Discover managed devices (`discoverManagedDevices`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: 

## <a name='applyManagedValues'></a>Apply managed values (`applyManagedValues`)
Type
: Slot

Allowed in states
: ACTIVE

Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Set all managed keys to the values currently active on the manager, replacing any manual change.

## <a name='managedKeys'></a>Managed keys (`managedKeys`)
Access mode
: RECONFIGURABLE

Assignment
: OPTIONAL

Description
: Properties and slots managed on devices in the pipeline.
