# cython: boundscheck=False
# cython: cdivision=True
# cython: wrapararound=False

cdef unsigned char NONE = 0
cdef unsigned char THRESHOLD = 1
cdef unsigned char OFFSET = 2
cdef unsigned char BLSHIFT = 4
cdef unsigned char REL_GAIN_PC = 8
cdef unsigned char GAIN_XRAY = 16
cdef unsigned char BPMASK = 32
cdef unsigned char FORCE_MG_IF_BELOW = 64
cdef unsigned char FORCE_HG_IF_BELOW = 128

from cython.parallel import prange

def correct(
    unsigned short[:, :, :, :] image_data,
    unsigned short[:] cell_table,
    unsigned char flags,
    unsigned char default_gain,
    float[:, :, :, :] threshold_map,
    float mg_hard_threshold,
    float hg_hard_threshold,
    float[:, :, :, :] offset_map,
    float[:, :, :, :] rel_gain_pc_map,
    float[:, :, :] md_additional_offset,
    float[:, :, :] rel_gain_xray_map,
    float g_gain_value,
    unsigned[:, :, :, :] bad_pixel_map,
    float bad_pixel_mask_value,
    # TODO: support spitting out gain map for preview purposes
    float[:, :, :] output
):
    cdef int frame, map_cell, x, y
    cdef unsigned char gain_stage
    cdef float raw_gain, res
    for frame in prange(image_data.shape[0], nogil=True):
        map_cell = cell_table[frame]
        if map_cell >= offset_map.shape[0]:
            for x in range(image_data.shape[1]):
                for y in range(image_data.shape[2]):
                    output[frame, x, y] = <float>image_data[frame, 0, x, y]
            continue
        for x in range(image_data.shape[2]):
            for y in range(image_data.shape[3]):
                gain_stage = default_gain
                res = image_data[frame, 0, x, y]
                raw_gain = image_data[frame, 1, x, y]
                if flags & THRESHOLD:
                    if raw_gain <= threshold_map[map_cell, x, y, 0]:
                        gain_stage = 0
                    elif raw_gain <= threshold_map[map_cell, x, y, 1]:
                        gain_stage = 1
                    else:
                        gain_stage = 2
                if (
                    flags & FORCE_MG_IF_BELOW
                    and gain_stage == 2
                    and (res - offset_map[map_cell, x, y, 1] < mg_hard_threshold)
                ):
                    gain_stage = 1
                if (
                    flags & FORCE_HG_IF_BELOW
                    and gain_stage == 1
                    and (res - offset_map[map_cell, x, y, 0] < hg_hard_threshold)
                ):
                    gain_stage = 0
                if flags & BPMASK and bad_pixel_map[map_cell, x, y, gain_stage] != 0:
                    res = bad_pixel_mask_value
                else:
                    if flags & OFFSET:
                        res = res - offset_map[map_cell, x, y, gain_stage]
                    if flags & REL_GAIN_PC:
                        res = res * rel_gain_pc_map[map_cell, x, y, gain_stage]
                        if gain_stage == 1:
                            res = res + md_additional_offset[map_cell, x, y]
                    if flags & GAIN_XRAY:
                        res = (res / rel_gain_xray_map[map_cell, x, y]) * g_gain_value
                output[frame, x, y] = res
