# cython: boundscheck=False
# cython: cdivision=True
# cython: wrapararound=False

cdef unsigned char NONE = 0
cdef unsigned char OFFSET = 1

from cython.parallel import prange

def correct(
    unsigned short[:, :, :] image_data,
    unsigned short[:] cell_table,
    unsigned char flags,
    float[:, :, :] offset_map,
    float[:, :, :] output
):
    cdef int frame, map_cell, x, y
    cdef float res
    for frame in prange(image_data.shape[0], nogil=True):
        map_cell = cell_table[frame]
        if map_cell >= offset_map.shape[0]:
            for x in range(image_data.shape[1]):
                for y in range(image_data.shape[2]):
                    output[frame, x, y] = <float>image_data[frame, x, y]
            continue
        for x in range(image_data.shape[1]):
            for y in range(image_data.shape[2]):
                res = image_data[frame, x, y]
                if flags & OFFSET:
                    res = res - offset_map[map_cell, x, y]
                output[frame, x, y] = res
