# cython: boundscheck=False
# cython: cdivision=True
# cython: wrapararound=False

cdef unsigned char NONE = 0
cdef unsigned char OFFSET = 1
cdef unsigned char GAIN_AMP = 2
cdef unsigned char REL_GAIN = 4
cdef unsigned char FF_CORR = 8
cdef unsigned char BPMASK = 16

from cython.parallel import prange
from libc.math cimport isinf, isnan

def correct(
    unsigned short[:, :, :, :] image_data,
    unsigned short[:] cell_table,
    unsigned char flags,
    float[:, :, :, :] offset_map,
    float[:, :, :, :] gain_amp_map,
    float[:, :, :, :] rel_gain_slopes_map,
    float[:, :, :, :] flatfield_map,
    unsigned[:, :, :, :] bad_pixel_map,
    float bad_pixel_mask_value,
    # TODO: support spitting out gain map for preview purposes
    float[:, :, :] output
):
    cdef int frame, map_cell, ss, fs
    cdef unsigned char gain_stage
    cdef float res
    cdef unsigned short raw_data_value
    for frame in prange(image_data.shape[0], nogil=True):
        map_cell = cell_table[frame]
        if map_cell >= offset_map.shape[0]:
            for ss in range(image_data.shape[1]):
                for fs in range(image_data.shape[2]):
                    output[frame, ss, fs] = <float>image_data[frame, 0, ss, fs]
            continue
        for ss in range(image_data.shape[1]):
            for fs in range(image_data.shape[3]):
                raw_data_value = image_data[frame, 0, ss, fs]
        gain_stage = (raw_data_value >> 12) & 0x0003
        res = <float>(raw_data_value & 0x0fff)
        if gain_stage > 2 or (flags & BPMASK and bad_pixel_map[map_cell, ss, fs, gain_stage] != 0):
            res = bad_pixel_mask_value
        else:
            if flags & OFFSET:
                res = res - offset_map[map_cell, ss, fs, gain_stage]
            if flags & GAIN_AMP:
                res = res * gain_amp_map[map_cell, ss, fs, gain_stage]
            if flags & FF_CORR:
                res = res * rel_gain_slopes_map[map_cell, ss, fs, gain_stage]
            if res < 1e-7 or res > 1e7 or isnan(res) or isinf(res):
                res = bad_pixel_mask_value
        output[frame, ss, fs] = res
